/**************************** prog-flash.asm *********************************

This program writes a .LDR file into the flash on ADDS-21161N-EZLITE. The .LDR
file should be in the "INCLUDE" format with a "HOST BUS WIDTH" (meaning the
word width) of 16.


LED status:

FLAG4 = LDR verified to fit in flash 		(Flash_File_Size OK)				
FLAG5 =	read manufacturer and device ID		(Flash_Auto_Select OK)
FLAG6 = FLASH erased 						(Flash_Chip_Erase OK)
FLAG7 = LDR Programming complete			(Flash_Write_Data OK)
FLAG8 = All data verified correct			(Flash_Verify_Data OK)
FLAG9 = ALL OK
 
*****************************************************************************/



#include    <def21161.h>


#define FlashStartAddress 0x4000000		//Flash starting address.  Mapped to MS1




.section/dm seg_sram;
.var input[1665]= "Blink.ldr"; //kernel in 16-bit include format


// Interrupt vector table - mapped to beginning of block0 (mem_ivt)
.section /pm   seg_rth;
Reserved_1:     nop; nop; nop; nop;
Chip_Reset:     idle; 
				jump start; 
				nop; 
				nop;


// program memory code 
.segment/pm seg_pmco;
start:
	
	r0 = 0;								// data to write to flash
	r1 = 0;								// data read from flash
	r2 = 0;								// mask value
	r3 = 0;								// Manufacturer ID read
	r4 = 0;								// Device ID read
	r5 = 0;								// data read from file
	r6 = 0;								// temp
	r7 = 0;								// timeout
	r8 = 0;								// polling1
	r9 = 0;								// polling2
	r10 = 0;							// finished
	r11 = 0;							// file flag
	r12 = 0;							// device flag
	r13 = 0;							// erase flag
	r14 = 0;							// write flag
	r15 = 0;							// verify flag
	
	l0 = 0;
	l7 = 0;
	
	bit clr mode1 CBUFEN;
	//set flags as outputs
	ustat1=dm(IOFLAG);
	bit set ustat1 FLG4O|FLG5O|FLG6O|FLG7O|FLG8O|FLG9O; 	
	dm(IOFLAG)=ustat1;

	//toggle flags (set to 1 = illuminated)
	lcntr=3, do (pause2 + 1) until LCE;
		bit set ustat1 FLG4|FLG5|FLG6|FLG7|FLG8|FLG9;
		dm(IOFLAG)=ustat1;
		lcntr = 0xFFF000; do pause1 until lce;	
			pause1:nop;
		bit clr ustat1 FLG4|FLG5|FLG6|FLG7|FLG8|FLG9;
		dm(IOFLAG)=ustat1;
		lcntr = 0xFFF000; do pause2 until lce;	
			pause2: nop;
	nop;
	


	call Flash_File_Size;				// Checks for proper file size
	call Flash_Reset;					// reset the flash device
	call Flash_Auto_Select;				// read manufacturer and device ID
	call Flash_Chip_Erase;				// erase the flash
	call Flash_Write_Data;				// write data to the flash
	call Flash_Verify_Data;				// verify correct data was written

	// all pass
	bit set ustat1 FLG9;
	dm(IOFLAG)=ustat1;
	jump finished;
//	idle;
	
Flash_Write_Data:
	r14 = 1;							// set error flag
	i0 = FlashStartAddress;				// set DAG to starting address of flash
	m0 = 1;								// set increment value
	m7 = 1;								// set increment value
	i7 = input;						// set DAG to starting address of payload-buffer
										// (plus one is because offset [from -p switch] is first word of .LDR

	r5=length(input);					// # 16bit words (see previous instruction/comment)
	r5=r5+r5;							// # 8-bit writes
	lcntr = r5;
	do Flash_Write - 1  until lce;		// do this loop until all the data has been written

		call Flash_Program_Unlock;		// unlock the flash so we can write data to it	
		r5 = dm(i7,m7);					// read 16 bits of valid data from the buffer
		dm(i0,m0) = r5;					// write the lower 8 bits of data to the flash
		call data_polling;				// check to see that the command completed

		r5 = lshift r5 by -8;			// shift data 8-bits to the right
		call Flash_Program_Unlock;		// unlock the flash so we can write data to it			
		dm(i0,m0) = r5;					// write the lower 8 bits of data to the flash
		call data_polling;	
		nop;
		nop;
		nop;			// check to see that the command completed
	Flash_Write:

	r14 = 2;							// clear error flag
	call Light_LEDs;					// light LED's to show status
	rts;								// return to where the function was called from



Flash_Verify_Data:
	r15 = 1;							// set error flag
	i0 = FlashStartAddress;				// set DAG to starting address of flash
	i1 = (input);						// set DAG to starting address of payload-buffer 
	lcntr = length(input);			// sets the number of reads
	r2 = 0xff;							// mask value
	do  ReadnCheck-1 until lce;			// do this loop until all the data has been verified
		r1 = dm(i0,m0);					// read a byte from flash
		r1 = r1 and r2;					// save valid portion
		r6 = r1;						// store in temp

		r1 = dm(i0,m0);					// read a second byte from flash
		r1 = r1 and r2;					// save valid portion
		r1 = lshift r1 by 8;			// shift valid portion left 8 bits
		r6 = r6 OR r1;					// store in temp
		r5 = dm(i1,m0);					// read 2 bytes from file
		comp (r5,r6);					// compare file against flash
		if ne jump error;
		nop;
		nop;
		nop;				// if not equal there was an error
ReadnCheck:		
	nop;								// used to flush the pipe
	r15 = 2;							// clear error flag
	call Light_LEDs;					// light LED's to show status
	rts;								// return to where the function was called from







Flash_File_Size:
	r11 = 1;							// set error flag
	r5 = r0;							// read the amount of bytes to program
	r6 = 524288;						// size of flash is 512K
	comp (r5, r6);
	if ge jump error;					// verify filesize is smaller than flash
	r11 = 2;							// clear error flag
	call Light_LEDs;					// light LED's to show status
	rts;

Flash_Reset:
	r0 = 0xaa;
	dm(FlashStartAddress + 0x555) = r0;	// 1st operation
	r0 = 0x55;
	dm(FlashStartAddress + 0x2aa) = r0;	//2nd operation
	r0 = 0xf0;
	dm(FlashStartAddress) = r0; 		//3rd operation
	rts;

Flash_Auto_Select:
	r12 = 1;							// set error flag
	r0 = 0xaa;
	dm(FlashStartAddress + 0x555) = r0;	// 1st operation
	r0 = 0x55;
	dm(FlashStartAddress + 0x2aa) = r0;	// 2nd operation
	r0 = 0x90;
	dm(FlashStartAddress + 0x555) = r0;	// 3rd operation
	r2 = 0xff;							// used to mask the byte of data read from the flash
	r3 = 0x20;  						// Manufacturer Code for ST Micro
	r4 = 0xe3;  						// Device Code for M29W040B
	r1 = dm(0x4000000);  				// reads the Manufacturer Code
	r1 = r1 and r2;
	comp (r1, r3);
	if ne jump error; 					// verify correct value read
	r1 = dm(0x4000001);  				// reads the Device Code
	r1 = r1 and r2;
	comp (r1, r4);
	if ne jump error;					// verify correct value read
	r12 = 2;							// clear error flag
	call Light_LEDs;					// light LED's to show status
	rts;

Flash_Chip_Erase:
	r13 = 1;  							// set error flag
	r0 = 0xaa;
	dm(FlashStartAddress + 0x555) = r0;	// 1st operation
	r0 = 0x55;
	dm(FlashStartAddress + 0x2aa) = r0;	// 2nd operation
	r0 = 0x80;
	dm(FlashStartAddress + 0x555) = r0;	// 3rd operation
	r0 = 0xaa;
	dm(FlashStartAddress + 0x555) = r0;	// 4th operation
	r0 = 0x55;
	dm(FlashStartAddress + 0x2aa) = r0;	// 5th operation
	r0 = 0x10;
	dm(FlashStartAddress + 0x555) = r0;	// 6th operation
	call data_polling;					// check to see that the command completed
	r13 = 2;							// clear error flag
	call Light_LEDs;					// light LED's to show status
	rts;								// return to where the function was called from

Flash_Program_Unlock:
	r0 = 0xaa;
	dm(FlashStartAddress + 0x555) = r0;	// 1st operation
	r0 = 0x55;
	dm(FlashStartAddress + 0x2aa) = r0; // 2nd operation
	r0 = 0xa0;
	dm(FlashStartAddress + 0x555) = r0; // 3rd operation
	rts;								// return to where the function was called from


data_polling:
	r7 = 0;								// clear timeout counter
	r6 = 0xffffff;  					// timeout value
	r2 = 0xff;							// mask value
	r8 = dm(FlashStartAddress);			// read the flash
	r9 = dm(FlashStartAddress);			// read the flash again
	r8 = r8 and r2;						// mask valid byte of data from flash
	r9 = r9 and r2;						// mask valid byte of data from flash
	comp(r8,r9);						// check if data toggled
	if eq rts;  						// if data didn't toggled then erase is finished
Check:
	r7 = r7 + 1;						// increment timeout counter 
	r8 = dm(FlashStartAddress);			// read the flash
	r9 = dm(FlashStartAddress);			// read the flash again
	r8 = r8 and r2;						// mask valid byte of data from flash
	r9 = r9 and r2;						// mask valid byte of data from flash
	comp(r8,r9);						// check if data toggled
	if eq rts;  						// if data didn't toggled then erase is finished
	comp (r6, r7);						// check to see if timeout reached
	if ne jump Check; 					// if timeout not reached check flash again
	jump error;							// erase did not finish in time
		
error:
	call Light_LEDs;
	jump error;  // if r13 = 1 failed in erase
				 // if r12 = 1 failed in write  i0 - 1 is the failing address
				 // if r11 = 1 failed on verify i0 - 1 is the failing address

finished:
	r10 = 2;
	call Light_LEDs;
	call waithere;
	jump finished;						// everything passed






Light_LEDs:
//----------------------------------------
File_Size:
	r6 = 1;
	comp(r11, r6);
	if ne jump Flag4_Pass;
Flag4_Fail:
	bit set ustat1 FLG4;
	dm(IOFLAG)=ustat1;
	call waithere;
	bit clr ustat1 FLG4;
	dm(IOFLAG)=ustat1;
	call waithere;
	jump Flag4_Fail;
Flag4_Pass:
	r6 = 2;
	comp(r11, r6);
	if ne jump Device;
	bit set ustat1 FLG4;
	dm(IOFLAG)=ustat1;

//----------------------------------------
Device:
	r6 = 1;
	comp(r12, r6);
	if ne jump Flag5_Pass;
Flag5_Fail:
	bit set ustat1 FLG5;
	dm(IOFLAG)=ustat1;
	call waithere;
	bit clr ustat1 FLG5;
	dm(IOFLAG)=ustat1;
	call waithere;
	jump Flag5_Fail;

Flag5_Pass:
	r6 = 2;
	comp(r12, r6);
	if ne jump Erase;
	bit set ustat1 FLG5;
	dm(IOFLAG)=ustat1;	
	
//----------------------------------------
Erase:
	r6 = 1;
	comp(r13, r6);
	if ne jump Flag6_Pass;
Flag6_Fail:
	bit set ustat1 FLG6;
	dm(IOFLAG)=ustat1;
	call waithere;
	bit clr ustat1 FLG6;
	dm(IOFLAG)=ustat1;
	call waithere;
	jump Flag6_Fail;

Flag6_Pass:
	r6 = 2;
	comp(r13, r6);
	if ne jump Write;
	bit set ustat1 FLG6;
	dm(IOFLAG)=ustat1;	
	
//----------------------------------------
Write:
	r6 = 1;
	comp(r14, r6);
	if ne jump Flag7_Pass;
Flag7_Fail:
	bit set ustat1 FLG7;
	dm(IOFLAG)=ustat1;
	call waithere;
	bit clr ustat1 FLG7;
	dm(IOFLAG)=ustat1;
	call waithere;
	jump Flag7_Fail;

Flag7_Pass:
	r6 = 2;
	comp(r14, r6);
	if ne jump Verify;
	bit set ustat1 FLG7;
	dm(IOFLAG)=ustat1;	
	
//----------------------------------------
Verify:
	r6 = 1;
	comp(r15, r6);
	if ne jump Flag8_Pass;
Flag8_Fail:
	bit set ustat1 FLG8;
	dm(IOFLAG)=ustat1;
	call waithere;
	bit clr ustat1 FLG8;
	dm(IOFLAG)=ustat1;
	call waithere;
	jump Flag8_Fail;

Flag8_Pass:
	r6 = 2;
	comp(r15, r6);
	if ne jump All_Pass;
	bit set ustat1 FLG8;
	dm(IOFLAG)=ustat1;

//----------------------------------------
All_Pass:
	r6 = 2;
	comp(r10, r6);
	if ne jump return;
	bit set ustat1 FLG9;
	dm(IOFLAG)=ustat1;
return:
	rts;

waithere:
	lcntr = 0x100000;
	do loophere until lce;
		nop;
		nop;
		nop;
loophere:
		nop;
		rts;

.endseg;



#ifdef TESTMACRO
.section/dm seg_dm16;
.var testvar=1234;
#endif

